// GoToSocial
// Copyright (C) GoToSocial Authors admin@gotosocial.org
// SPDX-License-Identifier: AGPL-3.0-or-later
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
//
// You should have received a copy of the GNU Affero General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

package migrations

import (
	"context"

	"github.com/superseriousbusiness/gotosocial/internal/log"
	"github.com/uptrace/bun"
)

func init() {
	up := func(ctx context.Context, db *bun.DB) error {
		return db.RunInTx(ctx, nil, func(ctx context.Context, tx bun.Tx) error {
			// Drop previous in_reply_to_account_id index.
			log.Info(ctx, "dropping previous statuses index, please wait and don't interrupt it (this may take a while)")
			if _, err := tx.
				NewDropIndex().
				Index("statuses_in_reply_to_account_id_idx").
				Exec(ctx); err != nil {
				return err
			}

			// Create new index to replace it, which also includes id DESC.
			log.Info(ctx, "creating new statuses index, please wait and don't interrupt it (this may take a while)")
			if _, err := tx.
				NewCreateIndex().
				Table("statuses").
				Index("statuses_in_reply_to_account_id_id_idx").
				Column("in_reply_to_account_id").
				ColumnExpr("id DESC").
				Exec(ctx); err != nil {
				return err
			}

			return nil
		})
	}

	down := func(ctx context.Context, db *bun.DB) error {
		return db.RunInTx(ctx, nil, func(ctx context.Context, tx bun.Tx) error {
			return nil
		})
	}

	if err := Migrations.Register(up, down); err != nil {
		panic(err)
	}
}
